@@ -2447,6 +2447,30 @@ IV hclassno(UV n) {
   return 12*h + ((b2*3 == n) ? 4 : square && !(n&1) ? 6 : 0);
 }
 
+UV polygonal_root(UV n, UV k, int* overflow) {
+  UV D, R;
+  MPUassert(k >= 3, "is_polygonal root < 3");
+  *overflow = 0;
+  if (n <= 1) return n;
+  if (k == 4) return is_perfect_square(n) ? isqrt(n) : 0;
+  if (k == 3) {
+    if (n >= UV_MAX/8) *overflow = 1;
+    D = n << 3;
+    R = 1;
+  } else {
+    if (k > UV_MAX/k || n > UV_MAX/(8*k-16)) *overflow = 1;
+    D = (8*k-16) * n;
+    R = (k-4) * (k-4);
+  }
+  if (D+R <= D) *overflow = 1;
+  D += R;
+  if (*overflow || !is_perfect_square(D)) return 0;
+  D = isqrt(D) + (k-4);
+  R = 2*k - 4;
+  if ((D % R) != 0) return 0;
+  return D/R;
+}
+
 /* These rank/unrank are O(n^2) algorithms using O(n) in-place space.
  * Bonet 2008 gives O(n log n) algorithms using a bit more space.
  *
@@ -2496,23 +2520,23 @@ int perm_to_num(int n, int *vec, UV *rank) {
   return 1;
 }
 
-void randperm(UV n, UV k, UV *S) {
+void randperm(void* ctx, UV n, UV k, UV *S) {
   UV i, j;
 
   if (k > n)  k = n;
 
   if        (k == 0) {
   } else if (k == 1) {
-    S[0] = urandomm64(n);
+    S[0] = urandomm64(ctx,n);
   } else if (n < ((BITS_PER_WORD==32) ? 13 : 21)) {
     int V[32];
-    num_to_perm(urandomm64(factorial(n)), n, V);
+    num_to_perm(urandomm64(ctx,factorial(n)), n, V);
     for (i = 0; i < k; i++)
       S[i] = V[i];
   } else if (k < n/5 && k < 1000) {   /* TODO: Improve this cutoff */
     for (i = 0; i < k; i++) {
       do {
-        S[i] = urandomm64(n);
+        S[i] = urandomm64(ctx,n);
         for (j = 0; j < i; j++)
           if (S[j] == S[i])
             break;
@@ -2522,7 +2546,7 @@ void randperm(UV n, UV k, UV *S) {
     for (i = 0; i < n; i++)
       S[i] = i;
     for (i = 0; i < k && i <= n-2; i++) {
-      j = urandomm64(n-i);
+      j = urandomm64(ctx,n-i);
       { UV t = S[i]; S[i] = S[i+j]; S[i+j] = t; }
     }
   }
