@@ -1,9 +1,7 @@
-#!/usr/bin/env perl
-use strict;
 use warnings;
 $| = 1;  # fast pipes
 
-use Math::Prime::Util;
+use Math::Prime::Util qw/urandomm/;
 use Math::Pari;
 
 my $nlinear = 100000;
@@ -30,7 +28,7 @@ print "Testing random numbers from $nlinear to ", $randmax, "\n";
 my $mod = int($nrandom / 80);
 
 while ($nrandom-- > 0) {
-  my $n = $nlinear + 1 + int(rand($randmax - $nlinear));
+  my $n = $nlinear + 1 + urandomm($randmax-$nlinear);
   die "failure for eulerphi($n)" unless Math::Prime::Util::euler_phi($n) == Math::Pari::eulerphi($n);
   die "failure for moebius($n)" unless Math::Prime::Util::moebius($n) == Math::Pari::moebius($n);
   print "." if ($nrandom % $mod) == 0;
